var APILogs = (function () {
  var Service = null;
  var APILogsData = [];
  var APILogsEditor = '';
  var currentCursorPosition = false;
  function init() {
    $('#dvPageContent').on('click', '#APILogsEditorSearch', function () {
      APILogsEditor.execCommand('find');
    });
    $('#dvPageContent').on('click', '#APILogsDownload', function () {
      downloadLogs();
    });

    Service = Services(
      PBConstants.BASE_API_URL,
      PBConstants.MARKET_CAP_API_URL,
      PBConstants.CURRENCIES_API_URL,
      PBConstants.GLOBAL_MARKET_CAP_API_URL
    );

    handleWindowResize();
  }

  function handleWindowResize() {
    var resized;
    $(window).on('resize orientationChanged', function () {
      clearTimeout(resized);
      resized = setTimeout(function () {
        if (APILogsEditor) {
          APILogsEditor.resize();
        }
      }, 400);
    });
  }

  function downloadLogs() {
    var logsContent = APILogsEditor.getValue();
    var filename = 'ptlog-' + DateTimeHelper.getCurrentDateForLogsFileName();
    var href = 'data:text/plain;charset=utf-8,' + encodeURIComponent(logsContent);
    var tmpLink = '<a class="logs-download hide" href="' + href + '" download="' + filename + '"></a>';
    $('body').append(tmpLink);
    var $logsDownloadLink = $('.logs-download');
    $logsDownloadLink[0].click();
    $logsDownloadLink.remove();
  }

  function callAPILogs() {
    $('#APILogsEditorLoading').removeClass('visibility-hidden').addClass('visibility-show');
    // If API logs call fails then ignore and show previous data.
    Service.getAPILogs().done(function (response) {
      normalizeAPILogsData(response);
      $('#APILogsEditorLoading').removeClass('visibility-show').addClass('visibility-hidden');
      currentCursorPosition = true;
    });
    // Call API logs for every x seconds.
    setTimeout(callAPILogs, PBConstants.API_LOGS_REFRESH);
  }

  function normalizeAPILogsData(currentAPILogsData) {
    // If API response is not array then show previous data.
    if ($.isArray(currentAPILogsData)) {
      APILogsData = currentAPILogsData;
    }
    setAPILogsContent(APILogsEditor, currentCursorPosition);
  }

  function setAPILogsContent(editor, currentCursorPositionParam) {
    if(!editor) {
      return;
    }
    APILogsEditor = editor;
    let APILogsDataString = APILogsData.join('\n');
    var editorScrollTop = APILogsEditor.session.getScrollTop();
    APILogsEditor.session.setValue(APILogsDataString, 1);
    // Logs editor should be readonly.
    APILogsEditor.setReadOnly(true);

    // Scrolling to end of the editor on page load and also when
    // user switches from another tab to Logs tab.
    // Not scrolling after API call to improve readability.
    if (currentCursorPositionParam) {
      APILogsEditor.session.setScrollTop(editorScrollTop);
    } else {
      // Ace eidtor gotoLine is not working in lower resolution so scrolling to bottom.
      if ($(window).innerWidth() < PBConstants.MIN_RESOLUTION_FOR_FIXED_HEADERS) {
        setTimeout(function () {
          $('html, body').animate({
            scrollTop: $('.footer').offset().top
          });
        }, 500);
      } else {
        setTimeout(function () {
          var EditorLength = APILogsEditor.session.getLength();
          APILogsEditor.gotoLine(EditorLength);
        }, 100);
      }

    }
  }

  return {
    init: init,
    callAPILogs: callAPILogs,
    setAPILogsContent: setAPILogsContent
  };
})();
